/***************************************************************************
 * ECE 4760 Final Project
 * 
 * Autonomous Drone
 * 
 * Ryan Hornung rmh286
 ***************************************************************************/

////////////////////////////////////
// clock AND protoThreads configure!
// You MUST check this file!
#include "config.h"
#include "config_bits.h"
// threading library
#include "pt_cornell_1_2.h"

////////////////////////////////////
#include "i2c_helper.h"
#include "LSM303.h"
#include "GPS.h"
#include "wifi.h"
#include "pwm.h"

////////////////////////////////////
// graphics libraries
#include "tft_master.h"
#include "tft_gfx.h"

////////////////////////////////////
#include <stdlib.h>
#include <math.h>

// === thread structures =======================================================
// thread control structs

// protothread structures
static struct pt pt_gps, pt_wifi, pt_compass ;
// The following threads are necessary for UART control
static struct pt pt_gps_input, pt_wifi_input, pt_output ;

/**************************************************************************/
/*!
    @brief  Timer2 interrupt handler. Deals with control of the drone
*/
/**************************************************************************/
void __ISR(_TIMER_2_VECTOR, ipl2) Timer2Handler(void)
{
    mT2ClearIntFlag();
    
    switch (flight) {
        case (take_off):
            SetDCOC1PWM(15593);
            if (PT_GET_TIME() > (take_off_start_time + take_off_duration)) {
                flight = hold;
            }
            break;
        case (land):
            SetDCOC1PWM(mid_rc);
            break;
        case (hold):
            SetDCOC1PWM(15550);
            SetDCOC3PWM(15200);
            break;
        case (kill):
            SetDCOC1PWM(rx_min_usec);
    }
     
} // end ISR TIMER2


/**************************************************************************/
/*!
    @brief  Protothread to read GPS data
*/
/**************************************************************************/
static PT_THREAD (protothread_gps(struct pt *pt)) {
    PT_BEGIN(pt);
    
        char buffer[128];
        
        while(1) {
        
            PT_SPAWN(pt, &pt_gps_input, PT_Get_GPS(&pt_gps_input));
            parse(PT_gps_buffer);
            
            PT_YIELD_TIME_msec(1000);
             
            // never exit while
      } // END WHILE(1)
  PT_END(pt);
}

/**************************************************************************/
/*!
    @brief  Protothread to read wifi data
*/
/**************************************************************************/
static PT_THREAD (protothread_wifi(struct pt *pt)) {
    PT_BEGIN(pt);
    
        char buffer[128];
        
        while(1) {
        
            PT_SPAWN(pt, &pt_wifi_input, PT_Get_WiFi(&pt_wifi_input));
            parse_wifi(PT_wifi_buffer);
            
            PT_YIELD_TIME_msec(1000);
             
            // never exit while
      } // END WHILE(1)
  PT_END(pt);
}

/**************************************************************************/
/*!
    @brief  Protothread to read compass data
*/
/**************************************************************************/
static PT_THREAD (protothread_compass(struct pt *pt)) {
    PT_BEGIN(pt);
    
        char buffer[128];
        
        LSM303_MAG_begin();
        
        while(1) {
        
            LSM303_MAG_get_event();
            get_heading();
            
            PT_YIELD_TIME_msec(1000);
             
            // never exit while
      } // END WHILE(1)
  PT_END(pt);
}

// === Main  ===================================================================
int main(void)
{
  
  // configure I2C
  i2c_init();
  // configure gps
  gps_init();
  // configure wifi
  wifi_init();
  // configure pwm signals
  pwm_init();
  
  OpenTimer2(T2_ON | T2_SOURCE_INT | T2_PS_1_1, 40000);
  // set up the timer interrupt with a priority of 2
  ConfigIntTimer2(T2_INT_ON | T2_INT_PRIOR_2);
  mT2ClearIntFlag(); // and clear the interrupt flag
  
  // === config the uart, DMA, vref, timer5 ISR ============
  PT_setup();

  // === setup system wide interrupts  =====================
  INTEnableSystemMultiVectoredInt();
  
  // === now the threads ===================================
  // init the threads
  PT_INIT(&pt_gps);
  PT_INIT(&pt_wifi);
  PT_INIT(&pt_compass);

  // schedule the threads
  while(1) {
      PT_SCHEDULE(protothread_gps(&pt_gps));
      PT_SCHEDULE(protothread_wifi(&pt_wifi));
      PT_SCHEDULE(protothread_compass(&pt_compass));
  }
} // main
