
#include "i2c_helper.h"

/**************************************************************************/
/*!
    @brief  Initialize I2C peripheral
*/
/**************************************************************************/

void i2c_init() {
    OpenI2C1( I2C_EN, BRG_VAL );
}


/**************************************************************************/
/*!
    @brief  Write using I2C protocol
*/
/**************************************************************************/

void i2c_write(char address, char sub_address, char *data, int num)
{
    char i2c_header[2];
    
    i2c_header[0] = address | I2C_WRITE;
	i2c_header[1] = sub_address;

    StartI2C1();
	IdleI2C1();

    int i;
	for(i = 0; i < num + 2; i++)
	{
        if(i < 2)
            MasterWriteI2C1( i2c_header[i] );
        else
            MasterWriteI2C1( data[i - 2] );
		IdleI2C1();

		//ACKSTAT is 0 when slave acknowledge, 
		//if 1 then slave has not acknowledge the data.
		if( I2C1STATbits.ACKSTAT )
			break;
	}
    
    StopI2C1();
	IdleI2C1();
}


/**************************************************************************/
/*!
    @brief  Read using the I2C protocol
*/
/**************************************************************************/

void i2c_read(char address, char sub_address, char *data, int num)
{
    char i2c_header[2];
    int i2c_header_size;
    
    i2c_header[0] = address | I2C_WRITE;
    i2c_header[1] = sub_address | I2C_AUTO_INC;
    i2c_header_size = 2;
    
    StartI2C1();
    IdleI2C1();
    
    int index = 0;
    while (i2c_header_size) {
        MasterWriteI2C1( i2c_header[index++] );
        IdleI2C1();

        i2c_header_size--;

        //ACKSTAT is 0 when slave acknowledge
        if (I2C1STATbits.ACKSTAT) {
            break;
        }
    }

    RestartI2C1();
    //i2c_wait(10);
    IdleI2C1();
    
    MasterWriteI2C1( address | I2C_READ );
    IdleI2C1();
    
    MastergetsI2C1(num, data, I2C_DATA_WAIT);
    IdleI2C1();
 
    StopI2C1();
    IdleI2C1();
}

